"""Functions for setting up a virtual environment for the installation."""
import os
import getpass
from pattoo_shared.installation import shared, configure
from pattoo_shared import log


def make_venv(file_path):
    """Create virtual environment for pattoo installation.

    Args:
        file_path: The path to the virtual environment

    Returns:
        None
    """
    # Say what we're doing
    print('??: Create virtual environment')
    command = 'python3 -m virtualenv {}'.format(file_path)
    shared.run_script(command)
    print('OK: Virtual environment created')
    # Ensure venv is owned by pattoo if the pattoo user exists
    if configure.user_exists('pattoo') and configure.group_exists('pattoo'):
        if getpass.getuser() == 'root':
            shared.run_script('chown -R pattoo:pattoo {}'.format(file_path))


def activate_venv(activation_path):
    """Activate the virtual environment in the current interpreter.

    Args:
        activation_path: The path to the activate_this.py file

    Returns:
        None

    """
    # Open activte_this.py for reading
    try:
        f_handle = open(activation_path)
    except PermissionError:
        log.log2die_safe(1061, '''\
Insufficient permissions on file: {}. Ensure the directory has the necessary \
read-write permissions'''.format(activation_path))
    except FileNotFoundError:
        log.log2die_safe(1074, '''\
activate_this.py for the venv has not been created at {} with virtualenv. \
Ensure that its created using the module "virtualenv"
'''.format(activation_path))
    else:
        with f_handle:
            code = compile(f_handle.read(), activation_path, 'exec')
            exec(code, dict(__file__=activation_path))


def environment_setup(file_path):
    """Create and activate virtual environment.

    Args:
        file_path: The path to the virtual environment

    Returns:
        None

    """
    # Initialize key variables
    activtion_path = os.path.join(file_path, 'bin/activate_this.py')

    make_venv(file_path)

    activate_venv(activtion_path)



#!/usr/bin/env python3
"""Test the encrypt module."""

# Standard imports
import unittest
import os
import sys
import shutil
import stat


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}test_pattoo_shared'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
import pattoo_shared.encrypt as pgp


class TestEncrypt(unittest.TestCase):
    """Test all methods of Pgpier class.
    """

    @classmethod
    def setUpClass(cls):
        """Class setup.

        Set up for the test class which generates the
        key pairs for twp Pgpier classes. This is done to
        faciliate the testing of the key exchange and
        public key encryption
        """

        # #####################Essential######################
        main_dir = os.path.abspath(
            os.path.join(
                os.getcwd(), '{}'.format('keys')
            )
        )

        cls.main_dir = main_dir
        # ###################################################

        # #####################1st Pgpier######################
        # Create sub directory for 1st Pgpier and change
        # permissions for the directory
        test1 = os.path.join(main_dir, '{}'.format('test1'))
        cls.test1 = test1
        person1 = os.path.join(test1, '{}'.format('.gnupg'))
        os.makedirs(person1)
        os.chmod(person1, stat.S_IRWXU)

        # Create Pgpier class
        gpg1 = pgp.Pgpier(person1)

        # Variables for 1st key pair generation
        # Some variables are saved in the class to
        # be accessed later.
        wrapper1 = '(Person1)'
        cls.wrapper1 = wrapper1
        person1_name = 'John Brown'
        person1_email = 'john_brown_2020_test@gmail.com'
        cls.person1_email = person1_email
        person1_comment = 'Unit testing with person1'

        # Checks if a passphrase has been exported already
        result1 = gpg1.set_from_imp(wrapper1)

        # If passphrase has not been export, a new key pair will be generated
        if not result1:
            gpg1.key_pair(person1_email, person1_name, person1_comment)
            gpg1.exp_main(wrapper1)
            cls.gpg1 = gpg1

        ####################################################

        # #####################2nd Pgpier######################
        # Create sub directory for 2nd Pgpier and change
        # permissions for the directory
        test2 = os.path.join(main_dir, '{}'.format('test2'))
        cls.test2 = test2
        person2 = os.path.join(test2, '{}'.format('.gnupg'))
        os.makedirs(person2)
        os.chmod(person2, stat.S_IRWXU)

        # Create Pgpier class
        gpg2 = pgp.Pgpier(person2)

        # Variables for 2nd key pair generation
        # Some variables are saved in the class to
        # be accessed later.
        wrapper2 = '(Person2)'
        cls.wrapper2 = wrapper2
        person2_name = 'Mary Jane'
        person2_email = 'mary_jane_2020_test@gmail.com'
        cls.person2_email = person2_email
        person2_comment = 'Unit testing with person2'

        # Checks if a passphrase has been exported already
        result2 = gpg2.set_from_imp(wrapper2)

        # If passphrase has not been export, a new key pair will be generated
        if not result2:
            gpg2.key_pair(person2_email, person2_name, person2_comment)
            gpg2.exp_main(wrapper2)
            cls.gpg2 = gpg2
        ####################################################

    @classmethod
    def tearDownClass(cls):
        """Class teardown.

        This will remove all test data by deleting the main
        generated directory
        """

        shutil.rmtree(cls.main_dir)

    def setUp(self):
        """This will run each time before a test is performed.
        """

        # Retrieves Pgpier object for each test Pgpier class
        self.gpg1 = self.__class__.gpg1
        self.gpg2 = self.__class__.gpg2

        # Retrieves directory for each test Pgpier class
        self.test1_dir = self.__class__.test1
        self.test2_dir = self.__class__.test2

        # Retrieves wrappers used to store passphrase for
        # each Pgpier class
        self.wrapper1 = self.__class__.wrapper1
        self.wrapper2 = self.__class__.wrapper2

        # Retrieves test email addresses of each Pgpier
        self.email_1 = self.__class__.person1_email
        self.email_2 = self.__class__.person2_email

    def tearDown(self):
        """This is performed after each test run.
        """

    def test_key_generation(self):
        """Checks if both Pgpier instances generated their key pairs.
        """

        # Gives initial False value
        # When unchanged, the test will fail
        key_gen1 = False
        key_gen2 = False

        # If a fingerprint was generated, a string value would be
        # assigned. If not, it would be an empty string
        if self.gpg1.fingerprint != '':
            key_gen1 = True

        if self.gpg2.fingerprint != '':
            key_gen2 = True

        # Retrieves public keys stored in each Pgpier object
        keys1 = self.gpg1.list_pub_keys()
        keys2 = self.gpg2.list_pub_keys()

        # Used to check if the keys were generated from the test
        # values assigned in the class set up
        person1_key = False
        person2_key = False

        # Checks if there are any public keys before searching
        if keys1 != []:
            for key in keys1:
                if key['uids'] == ['John Brown (Unit testing '
                                   'with person1) <john_brown'
                                   '_2020_test@gmail.com>']:
                    person1_key = True

        if keys2 != []:
            for key in keys2:
                if key['uids'] == ['Mary Jane (Unit testing '
                                   'with person2) <mary_jane'
                                   '_2020_test@gmail.com>']:
                    person2_key = True

        # Checks if the keys were generated from the specific
        # test values used in the class set up
        self.assertTrue(person1_key)
        self.assertTrue(person2_key)

        # Checks if any key pairs were generated at all
        self.assertTrue(key_gen1)
        self.assertTrue(key_gen2)

    def test_set_passphrase(self):
        """Test if set passphrase method works.
        """

        # Retrieves already set passphrases
        prev_passphrase1 = self.gpg1.passphrase
        prev_passphrase2 = self.gpg2.passphrase

        # Values to be used for next passphrase
        next_passphrase1 = 'password-123'
        next_passphrase2 = 'password-abc'

        # Assignment of new passphrases
        self.gpg1.set_passphrase(next_passphrase1)
        self.gpg2.set_passphrase(next_passphrase2)

        # Retrieves set passphrase from Pgpier objects
        set_passphrase1 = self.gpg1.passphrase
        set_passphrase2 = self.gpg2.passphrase

        # Checks if new passphrases were assigned
        self.assertEqual(set_passphrase1, next_passphrase1)
        self.assertEqual(set_passphrase2, next_passphrase2)

        # Resets to previous passphrases and check it they
        # were changed back
        self.gpg1.set_passphrase(prev_passphrase1)
        self.gpg2.set_passphrase(prev_passphrase2)

        set_passphrase1 = self.gpg1.passphrase
        set_passphrase2 = self.gpg2.passphrase

        self.assertEqual(set_passphrase1, prev_passphrase1)
        self.assertEqual(set_passphrase2, prev_passphrase2)

    def test_set_keyid(self):
        """Checks if set key ID methods works."""

        # Sets key ID's for each Pgpier object
        self.gpg1.set_keyid()
        self.gpg2.set_keyid()

        keyid1 = self.gpg1.keyid
        keyid2 = self.gpg2.keyid

        # Checks that the key ID's are not empty
        self.assertIsNotNone(keyid1)
        self.assertIsNotNone(keyid2)

    def test_list_pub_keys(self):
        """Test that public keys are listed
        """

        # List all public keys
        keys_lst1 = self.gpg1.list_pub_keys()
        keys_lst2 = self.gpg2.list_pub_keys()

        # Used to check that the lists are not empty
        is_keys_lst1 = False
        is_keys_lst2 = False

        if keys_lst1 != []:
            is_keys_lst1 = True

        if keys_lst2 != []:
            is_keys_lst2 = True

        # If the lists are not empty, the test will pass
        self.assertTrue(is_keys_lst1)
        self.assertTrue(is_keys_lst2)

    def test_exp_main(self):
        """Export main test.

        Checks that the file used to store the passphrase
        of the main public private key pair is created
        """

        # Used to determine if the test passes
        pass_result1 = False
        pass_result2 = False

        # New wrappers to export new files of Pgpier
        # objects that already have a key pair
        wrapper1 = '(ExportTest_1)'
        wrapper2 = '(ExportTest_2)'

        # Retrieves fingerprint which is the first part
        # of the exported file name and, the passphrase
        # which will be stored inside the files
        fingerprint1 = self.gpg1.fingerprint
        fingerprint2 = self.gpg2.fingerprint
        passphrase1 = self.gpg1.passphrase
        passphrase2 = self.gpg2.passphrase

        # Export the information
        self.gpg1.exp_main(wrapper1)
        self.gpg2.exp_main(wrapper2)

        # Checks that the files in the current directory has the
        # fingerprints and checks if the wrappers are also at the
        # end of the file and stores the results in a list
        files1 = [f for f in os.listdir(self.test1_dir)
                  if f.endswith(wrapper1) and fingerprint1 in f]
        files2 = [f for f in os.listdir(self.test2_dir)
                  if f.endswith(wrapper2) and fingerprint2 in f]

        # Checks that the files were found
        if files1 != [] and files2 != []:

            # Opens and checks contents of the files
            with open('{}{}{}'.format(self.test1_dir, os.sep, files1[0]),
                      '{}'.format('r')) as f1:
                contents1 = f1.read()
                if contents1 == passphrase1:
                    # Assigns passing value if intended contents were stored
                    pass_result1 = True

            with open('{}{}{}'.format(self.test2_dir, os.sep, files2[0]),
                      '{}'.format('r')) as f2:
                contents2 = f2.read()
                if contents2 == passphrase2:
                    pass_result2 = True

        # Passes if the passing values were assigned
        self.assertTrue(pass_result1)
        self.assertTrue(pass_result2)

    def test_set_from_imp(self):
        """Tests if the setter for the export file work.
        """
        result1 = self.gpg1.set_from_imp(self.wrapper1)
        result2 = self.gpg2.set_from_imp(self.wrapper2)

        # The results will be true if the export files were found
        self.assertTrue(result1)
        self.assertTrue(result2)

    def test_exp_pub_key(self):
        """Export public key test.

        Tests that the main public key of the Pgpier
        class is exported
        """

        # Sets key ID used to export public keys
        self.gpg1.set_keyid()
        self.gpg2.set_keyid()

        # Exports public keys
        pub_key1 = self.gpg1.exp_pub_key()
        pub_key2 = self.gpg2.exp_pub_key()

        # Test passes if the outputs are not None
        self.assertIsNotNone(pub_key1)
        self.assertIsNotNone(pub_key2)

    def test_imp_pub_key(self):
        """Tests that the importation of public keys work.
        """

        # Sets key ID's used to export public keys
        self.gpg1.set_keyid()
        self.gpg2.set_keyid()

        keys_lst1 = self.gpg1.list_pub_keys()
        keys_lst2 = self.gpg2.list_pub_keys()

        # Export public keys
        pub_key1 = self.gpg1.exp_pub_key()
        pub_key2 = self.gpg2.exp_pub_key()

        # Import public keys
        result_1 = self.gpg1.imp_pub_key(pub_key2)
        result_2 = self.gpg2.imp_pub_key(pub_key1)

        # Retrieves new list of public keys
        keys_lst1 = self.gpg1.list_pub_keys()
        keys_lst2 = self.gpg2.list_pub_keys()

        # Checks that each Pgpier object retrieve the other's
        # public key
        self.assertEqual(len(keys_lst1), 2)
        self.assertEqual(len(keys_lst2), 2)

    def test_email_to_key(self):
        """Email to fingerprint test.

        Test if the fingerprint is retrieved for a public key
        that has an assigned email address"""

        # Retrieves email addresses from class set up
        email1 = self.__class__.person1_email
        email2 = self.__class__.person2_email

        # Retrieves fingerprints from email addresses
        fp1 = self.gpg1.email_to_key(email1)
        fp2 = self.gpg2.email_to_key(email2)

        self.assertIsNotNone(fp1)
        self.assertIsNotNone(fp2)

    def test_encrypt_decrypt_data(self):
        """Test both public key encryption and decryption.
        """

        original_data = 'HELLO WORLD!'

        # Set key id's
        self.gpg1.set_keyid()
        self.gpg2.set_keyid()

        # Export public key to ASCII string
        pub_key1 = self.gpg1.exp_pub_key()
        pub_key2 = self.gpg2.exp_pub_key()

        # Import public key
        self.gpg1.imp_pub_key(pub_key2)  # Trade
        self.gpg2.imp_pub_key(pub_key1)  # Trade

        # Get email from class setup
        email1 = self.__class__.person1_email
        email2 = self.__class__.person2_email

        # Get fingerprint from email address
        fp1 = self.gpg1.email_to_key(email1)
        fp2 = self.gpg2.email_to_key(email2)

        # Both keys have to be trusted before data can be encrypted using an
        # imported public key

        # Trust public keys so that encryption and decryption can happen
        self.gpg1.trust_key(fp2)  # Trade
        self.gpg2.trust_key(fp1)  # Trade

        # Encrypt data from 1st Pgpier using 2nd Pgpier's public key
        encrypted_data = self.gpg1.encrypt_data(original_data, fp2)

        # Decrypt data using 2nd Pgpier's passphrase to access the keyring
        passphrase2 = self.gpg2.passphrase
        decrypted_data = self.gpg2.decrypt_data(encrypted_data, passphrase2)

        # Checks that the decrypted data is the same as the original
        self.assertEqual(original_data, decrypted_data)

    def test_gen_symm_key(self):
        """Generate symmetric key test.

        Checks the default length of random string
        generation"""

        string1 = self.gpg1.gen_symm_key()
        # string2 = self.gpg2.gen_symm_key()

        # Checks that the length of the generated data is the same
        # as the default and that the data is a string
        self.assertEqual(len(string1), 70)
        self.assertEqual(type(string1), str)

    def test_symmetric_encrypt_decrypt(self):
        """Tests both symmetric encryption and decryption.
        """

        original_data = 'HELLO WORLD!'

        # Generates random symmetric key
        password = self.gpg1.gen_symm_key()

        # Encrypts data
        encrypted_data = self.gpg1.symmetric_encrypt(original_data, password)

        # Decrypts data
        decrypted_data = self.gpg2.symmetric_decrypt(encrypted_data, password)

        self.assertEqual(original_data, decrypted_data)

    def test_set_email(self):
        """Set email test.

        Tests that the email address in the Pgpier class is
        set from the key pair"""

        # Sets email addresses
        self.gpg1.set_email()
        self.gpg2.set_email()

        # Retrieves email addresses in Pgpier objects
        set_email1 = self.gpg1.email_addr
        set_email2 = self.gpg2.email_addr

        # Checks that the assigned emails from the class set up
        # is the same as the ones stored in the Pgpier classes
        self.assertEqual(self.email_1, set_email1)
        self.assertEqual(self.email_2, set_email2)


if __name__ == '__main__':
    unittest.main()



"""Functions for installing external packages."""
# Standard imports
import os
import getpass

# Import pattoo related libraries
from pattoo_shared.installation import shared
from pattoo_shared import log


def install_missing_pip3(package, verbose=False):
    """Automatically Install missing pip3 packages.

    Args:
        package: The pip3 package to be installed

    Returns:
        True: if the package could be successfully installed

    """
    # Validate pip directory
    shared.run_script('''\
python3 -m pip install {0} -U --force-reinstall'''.format(package), verbose=verbose)


def install(requirements_dir, installation_directory, verbose=False):
    """Ensure PIP3 packages are installed correctly.

    Args:
        requirements_dir: The directory with the pip_requirements file.
        installation_directory: Directory where packages must be installed.
        verbose: Print status messages if True

    Returns:
        True if pip3 packages are installed successfully

    """
    # Initialize key variables
    lines = []

    # Read pip_requirements file
    filepath = '{}{}pip_requirements.txt'.format(requirements_dir, os.sep)

    # Say what we are doing
    print('Checking pip3 packages')
    if os.path.isfile(filepath) is False:
        shared.log('Cannot find PIP3 requirements file {}'.format(filepath))

    # Opens pip_requirements file for reading
    try:
        _fp = open(filepath, 'r')
    except PermissionError:
        log.log2die_safe(1079, '''\
Insufficient permissions for reading the file: {}. \
Ensure the file has read-write permissions and try again'''.format(filepath))
    else:
        line = _fp.readline()
        while line:
            # Strip line
            _line = line.strip()
            # Read line
            if True in [_line.startswith('#'), bool(_line) is False]:
                pass
            else:
                lines.append(_line)
            line = _fp.readline()

    # Process each line of the file
    for line in lines:
        # Determine the package
        package = line.split('=', 1)[0]
        package = package.split('>', 1)[0]

        # If verbose is true, the package being checked is shown
        if verbose:
            print('Installing package {}'.format(package))
        command = 'python3 -m pip show {}'.format(package)
        (returncode, _, _) = shared.run_script(
            command, verbose=verbose, die=False)

        # Install any missing pip3 package
        if bool(returncode) is True:
            install_missing_pip3(package, verbose=verbose)

    # Set ownership of any newly installed python packages to pattoo user
    if getpass.getuser() == 'root':
        if os.path.isdir(installation_directory) is True:
            shared.run_script('chown -R pattoo:pattoo {}'.format(
                installation_directory), verbose=verbose)

    print('pip3 packages successfully installed')



#!/usr/bin/env python3
"""Test the daemon module."""

# Standard imports
import unittest
import os
import subprocess
import sys
from io import StringIO
from unittest.mock import patch


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}test_pattoo_shared'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared import files
from pattoo_shared.daemon import Daemon, GracefulDaemon
from pattoo_shared.agent import Agent
from pattoo_shared.configuration import Config
from tests.libraries.configuration import UnittestConfig

# TEST CONSTANTS
AGENT_NAME = 'parent'
_config = Config()

def handle_daemon(command):
    """Allows for the daemon start

    Args:
        command: either --start/--restart to test

    Return:
        None

    """
    daemon_start_script_path = os.path.join(EXEC_DIR, 'daemon_start_test_script.py')
    subprocess.call(['python', daemon_start_script_path, command])

def create_agent():
    """Creates new agent for use in testing start and restart

    Args:
        None

    Return:
        _agent: new agent for testing

    """
    _agent = Agent(parent=AGENT_NAME, config=_config)
    return _agent

class MockDaemonMixin():
    """Mixin definging run functoin for MockDaemon and MockGracefulDaemon"""

    def run(self, loop=True):
        """Overriding Daemon run method

        Prints to standard output

        Args:
            loop: determines looping functionality is needed, used to test
            daemon start

        Return:
            None

        """
        print('Running')
        while loop:
            pass


class MockDaemon(MockDaemonMixin, Daemon):
    """Mock Daemon used to test Daemon class

    Built to provide minimal functionality to test Daemon run method

    """

class MockGracefulDaemon(MockDaemonMixin, GracefulDaemon):
    """Mock Graceful Daemon used to test Graceful Daemon class

    Built to provide minimal functionality to test Graceful Daemon run method

    """

class TestDaemon(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def setUp(self):
        """Test setup"""

        # Setup base config and agent
        self._agent = create_agent()
        self._config = _config

        # Instantiation of test daemon
        self._daemon = MockDaemon(self._agent)

    def tearDown(self):
        """Test clean up"""
        self._config = None
        self._agent = None

        # Removing any relate lock file
        if os.path.exists(self._daemon.lockfile):
            os.remove(self._daemon.lockfile)
        if os.path.exists(self._daemon.pidfile):
            os.remove(self._daemon.pidfile)

        self._daemon = None

    def test___init__(self):
        """Testing function __init__."""
        # Check daemon name matches agent name
        self.assertEqual(self._daemon.name, AGENT_NAME)

        # Checking daemon config
        self.assertEqual(self._daemon._config, self._config)
        self.assertEqual(self._daemon._config, self._config)

        # Checking daemon pid_file
        expected = files.pid_file(AGENT_NAME, self._config)
        self.assertEqual(self._daemon.pidfile, expected)

        # Checking daemon lock_file
        expected = files.lock_file(AGENT_NAME, self._config)
        self.assertEqual(self._daemon.lockfile, expected)

    def test__daemonize(self):
        """Testing function _daemonize."""
        pass

    def test_delpid(self):
        """Testing function delpid."""

        # Creating daemon Process ID file(pidfile)
        os.mknod(self._daemon.pidfile)

        # Checking that pid file has been created
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Delete pid file
        self._daemon.delpid()

        # Check that pidfile of the daemon has been deleted
        self.assertFalse(os.path.exists(self._daemon.pidfile))

    def test_dellock(self):
        """Testing function dellock."""

        # Creating daemon Process ID file(pidfile)
        os.mknod(self._daemon.lockfile)

        # Checking that pid file has been created
        self.assertTrue(os.path.exists(self._daemon.lockfile))

        # Delete pid file
        self._daemon.dellock()

        # Check that pidfile of the daemon has been deleted
        self.assertFalse(os.path.exists(self._daemon.lockfile))

    def test_start(self):
        """Testing function start."""
        handle_daemon('--start')
        self.assertTrue(os.path.exists(self._daemon.pidfile))

    def test_force(self):
        """Testing function force."""

        # Staring Daemon
        handle_daemon('--start')
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Calling force stop
        self._daemon.force()
        self.assertFalse(os.path.exists(self._daemon.pidfile))

    def test_stop(self):
        """Testing function stop."""

        # Staring Daemon
        handle_daemon('--start')
        self.assertTrue(os.path.exists(self._daemon.pidfile))
        os.mknod(self._daemon.lockfile)

        # Calling force stop
        self._daemon.stop()
        self.assertFalse(os.path.exists(self._daemon.pidfile))
        self.assertFalse(os.path.exists(self._daemon.lockfile))

    def test_restart(self):
        """Testing function restart."""

        # Creating daemon pidfile
        with open(self._daemon.pidfile, 'w') as f:
            f.write('99999')
        self.assertTrue(os.path.exists(self._daemon.pidfile))

        # Checking daemon restarting, pidfile should exist
        handle_daemon('--restart')
        self.assertTrue(os.path.exists(self._daemon.pidfile))

    def test_status(self):
        """Testing function status."""

        # Test status while daemon is running
        handle_daemon('--start')
        expected = 'Daemon is running - {}\n'.format(AGENT_NAME)

        with patch('sys.stdout', new = StringIO()) as result:
            self._daemon.status()
            self.assertEqual(result.getvalue(), expected)

        # Test status when daemon has been stopped
        os.remove(self._daemon.pidfile)
        expected = 'Daemon is stopped - {}\n'.format(AGENT_NAME)

        with patch('sys.stdout', new = StringIO()) as result:
            self._daemon.status()
            self.assertEqual(result.getvalue(), expected)

    def test_run(self):
        """Testing function run."""
        expected = 'Running\n'
        with patch('sys.stdout', new = StringIO()) as result:
            self._daemon.run(loop=False)
            self.assertEqual(result.getvalue(), expected)

class TestGracefulDaemon(TestDaemon):
    """

    Checks that daemon start/stop commands confirm to graceful shutdown

    """

    def setUp(self):
        """Test setup"""

        # Setup base config and agent
        self._agent = create_agent()
        self._config = _config

        # Instantiation of test daemon
        self._daemon = MockGracefulDaemon(self._agent)

    def graceful_fn(self, callback):
        """Sets up and executes test callback that should implement graceful
        shutdown

        Args:
            callback: function that implements graceful shutdown functionality

        Return:
            wrapper: implements setup before using callback and making
            assertions

        """
        def wrapper():
            """Wrapper function to be returned by graceful_fn"""
            # Testing proper graceful shutdown by creating lock file to simulate
            # that a process is currently handling data.
            handle_daemon('--start')
            os.mknod(self._daemon.lockfile)
            self.assertTrue(os.path.exists(self._daemon.lockfile))
            callback()

            # Checking that both daemon pidfile and lockfile do not exist, which
            # indicates successful stoppage of daemon.
            self.assertFalse(os.path.exists(self._daemon.lockfile))
            self.assertFalse(os.path.exists(self._daemon.pidfile))

        return wrapper

    def test_stop(self):
        """Testing graceful stop function"""

        # Test base Daemon stop functionality
        # When lock file does not exist
        super(TestGracefulDaemon, self).test_stop()

        # Graceful stop testing
        self.graceful_fn(self._daemon.stop())

    def test_restart(self):
        """Testing graceful restart function"""

        # Test base Daemon restart functionality
        # When lock file does not exist
        super(TestGracefulDaemon, self).test_restart()

        # Graceful stop testing
        self.graceful_fn(handle_daemon('--restart'))

if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3

import gnupg
import uuid
import hashlib
import os
import random
import string
import stat


class Pgpier:
    """Pgpier class.

    A class that handles encryption and decryption
    using the python-gnupg module

    gpg, GPG, Pgpier and GnuPG is used interchangeably as the Pgpier
    class outside this file.

    Resources:

    Chapter 7. Kurose J., Ross K.. (April, 2016).
    Computer Networking: A Top Down Approach.
    Pearson/Addison Wesley. Retrieved from
    http://www-net.cs.umass.edu/kurose-ross-ppt-7e/Chapter_8_V7.0.pptx

    Secure email style of encryption was used. So first, the key pairs
    are generated (public and private keys) so that it can be used to
    encrypt a randomly generated nonce and symmetric key. The nonce is
    used to provide a level of authentication. The symmetric key is
    used to encrypt the data since it is computationally less expensive
    than encrypting with a public key.

    What makes this module special, is it's ability to have a
    non-volatile approach to handle the passphrase, fingerprint
    and other variables with ease. The python-gnupg module
    provides most of the functionalities except keeping a track
    of the passphrase, the fingerprint, who owns the
    passphrase and fingerprint, etc. This module also simplifies
    the python-gnupg module to it can be widely used.

    The module uses one primary public private key pair to encrypt
    data. However, the module can also store other public keys.

    """

    def __init__(self, working_dir):
        """Instantiation of the class.

        The Pgpier class will instantiate with the working directory
        that also has a parent directory

        Args:
            working_dir (str): The path to the .gnupg directory where
                               GnuPG files will be stored
                               with a parent directory where this class
                               will store and handle pertinent
                               data

        Returns:
            None
        """

        self.wrk_dir = os.path.abspath(
            os.path.join(working_dir, os.pardir)
            )  # gets the parent of the working directory
        self.gnupghome = working_dir
        # , options=['--pinentry-mode=loopback']
        # the loopback option makes sure the passphrase can
        # be entered via python and won't prompt the user
        self.gpg = gnupg.GPG(gnupghome=working_dir,
                             options=['--pinentry-mode=loopback'])
        self.gpg.encoding = 'utf-8'  # sets encoding
        self.passphrase = None
        self.fingerprint = None
        self.keyid = None

    def key_pair(self, _name_email, _name_real,
                 _name_comment="auto generated using gnupg.py",
                 _key_type="RSA", _key_length=4096):
        """The generation of the private public key pairs.

        Args:
            _name_email (str): Email of the user
            _name_real (str): Full name of the user
            _name_comment (str): Optional comment for the user
            _key_type (str): The key type of the private public key pair
            _key_length (int): Key length of the private public key pair

        Returns:
            None
        """
        # generates random passphrase
        self.passphrase = hashlib.sha256(str(uuid.uuid4())
                                         .encode()).hexdigest()
        # helper method to get key configuration
        input_data = self.gpg.gen_key_input(key_type=_key_type,
                                            key_length=_key_length,
                                            name_real=_name_real,
                                            name_comment=_name_comment,
                                            name_email=_name_email,
                                            passphrase=self.passphrase)
        # generation of key pair
        key = self.gpg.gen_key(input_data)
        # print("stderr: ", key.stderr)
        self.fingerprint = key.fingerprint  # store fingerprint in class

    def set_passphrase(self, passphrase):
        """Method to set the passphrase in the class.

        The passphrase is needed to decrypt data encrypted
        by a public key.

        Args:
            passphrase (str): Passphrase to store in class

        Returns:
            None
        """
        self.passphrase = passphrase

    def set_fingerprint(self, fingerprint):
        """Method to set fingerprint in the class.

        Args:
            fingerprint (str): Fingerprint of the public private key pair

        Returns:
            None
        """
        self.fingerprint = fingerprint

    def set_keyid(self):
        """Set keyid in object.

        Method to set keyid by retrieving all keys stored
        in the GnuPG keyring and retrieve
        the keyid associated with the main public private key pair

        Args:
            None

        Returns:
            None
        """

        keys = self.list_pub_keys()
        fingerprint = self.fingerprint

        if keys != []:
            for key in keys:
                if key['fingerprint'] == fingerprint:
                    # set keyid associated with fingerprint in class
                    self.keyid = key['keyid']
        else:
            pass

    def list_pub_keys(self):
        """Method to list all the public keys stored in the GnuPG keyring.

        Args:
            None

        Returns:
            list: List of dictionaries of each public key stored
        """
        public_keys = self.gpg.list_keys()
        return public_keys

    def exp_main(self, _wrapper='(main)'):
        """Export pertinent information.

        Method to store the passphrase for future retrieval
        and name the file by the fingerprint of the
        class. The method also adds a wrapper to the
        name of the file so that when the Pgpier class is looking
        for the public private key pair, it finds the pair it owns.

        Args:
            _wrapper (str): The name of the wrapper

        Returns:
            None
        """
        # lists all files existing in a dir and checks if
        # the file ends with the wrapper
        _path = self.wrk_dir
        _filename = self.fingerprint
        # _wrapper = '(main)'
        _contents = self.passphrase

        file_names = [
            file_wrapper for file_wrapper in os.listdir(_path)
            if os.path.isfile(file_wrapper) and
            file_wrapper.endswith(_wrapper)
            ]
        if file_names != []:
            for f_name in file_names:

                # removes the wrapper
                file_name_len = len(f_name)
                wrapper_len = len(_wrapper)
                file_nowrap = file_name_len - wrapper_len
                clean_f_name = f_name[0:file_nowrap]

                # clean file name
                clean_f = os.path.abspath(os.path.join(_path, clean_f_name))
                # implement so that if the file already
                # exists it would make a copy
                try:
                    # renames the file without the wrapper
                    os.rename(f_name, clean_f)
                except Exception as e:
                    print(e)

        _file = os.path.abspath(os.path.join(_path, '{0}{1}'
                                .format(_filename, _wrapper)))
        with open('{}'.format(_file), 'w') as file_export:
            file_export.write(_contents)

        # Change filemode to 600
        # Only allow the user to access exported passphrase
        os.chmod(_file, stat.S_IRUSR | stat.S_IWUSR)

    def imp_main(self, _wrapper='(main)'):
        """Import pertinent information.

        Method to import the fingerprint and passphrase of the owned
        public private key pair of the
        user. The method also looks for a wrapper on the file to
        distinguish the public private key
        pair the user currently owns.

        Args:
            _wrapper (str): The name of the wrapper

        Returns:
            tuple: String of fingerprint and string of passphrase
            if it finds the public private key pairs
            the user currently owns

            None: If there are no public private key pair the user
            currently owns
        """

        # path to parent directory of gnupg home
        # where Pgpier will operate its own files
        _path = self.wrk_dir
        # _wrapper = '(main)'

        # returns list of files if it ends with the wrapper
        key = [_file for _file in os.listdir(_path)
               if _file.endswith(_wrapper)]

        key_len = len(key)
        if key_len > 1:
            raise Exception("critical error - 0: more than one main"
                            " keys\nreport issue")

        elif key_len == 1:
            _fingerprint = key[0]

            # removes the wrapper
            file_name_len = len(_fingerprint)
            wrapper_len = len(_wrapper)
            file_nowrap = file_name_len - wrapper_len
            clean_fp = _fingerprint[0:file_nowrap]

            fp_file = os.path.abspath(os.path.join(_path, _fingerprint))

            with open('{}'.format(fp_file), '{}'.format('r')) as f:
                _passphrase = f.read()

            if type(_passphrase) != str:
                raise Exception("critical error - 1: error reading for "
                                "passphrase\nreport issue")

            return clean_fp, _passphrase

        else:
            return None

    def set_from_imp(self, wrapper='(main)'):
        """Set object variables from import.

        Method to get the fingerprint and passphrase the user
        currently owns and then assign those values inside
        the class to utilize the user's public private key pair

        Args:
            None

        Returns:
            True: If it retrieved the fingerprint and passphrase from a file
            False: If it did not retrieve anything
        """
        result = None
        try:
            result = self.imp_main(wrapper)
        except Exception as e:
            print(e)

        if result is not None:
            self.set_fingerprint(result[0])
            self.set_passphrase(result[1])

        success = True if result is not None else False
        return success

    def exp_pub_key(self):
        """Method to export the user's public key into ASCII.

        Args:
            None

        Returns:
            str: String of ASCII armored public key
            None: If keyid is not set
        """
        ascii_armored_public_keys = None
        keyid = self.keyid
        gpg = self.gpg

        if keyid is not None:
            ascii_armored_public_keys = gpg.export_keys(keyid)
            return ascii_armored_public_keys
        else:
            return ascii_armored_public_keys

    def imp_pub_key(self, key_data):
        """Import public key.

        Method to import the ASCII public of a user
         into the current user's GnuPG keyring

        Args:
            key_data (str): String of public key armored ASCII

        Returns:
            result (dict): The number of imported keys and the
                           the number of keys not imported
        """
        gpg = self.gpg

        import_result = gpg.import_keys(key_data)

        # Returns the amount of: imported, not imported
        result = {"imported": import_result.imported,
                  "not_imported": import_result.not_imported}
        return result

    def pub_file(self):
        """Method to export the armored ASCII public key into an asc file.

        Args:
            None

        Returns:
            None
        """

        pub_key = self.exp_pub_key()
        fingerprint = self.fingerprint
        path = self.wrk_dir
        # export to parent directory of gnupg home
        pub_file = os.path.abspath(os.path.join(path, fingerprint))
        # checks that the class' public key was exported
        if pub_key is not None:
            with open('{0}{1}'.format(pub_file, '.asc'),
                      '{}'.format('w')) as f:
                f.write(pub_key)

    def sym_encrypt_files(self, symmetric_key, file_path, output,
                          delaf=False, algorithm='AES256', armor=True):
        """Method to encrypt files using a symmetric key.

        Args:
            symmetric_key (str): String of passphrase to be
            used to encrypt the data
            file_path (str): Absolute file path to the files to be encrypted
            output (str): Absolute file path to intended file output
            delaf (bool): True if the files should be deleted after encryption
                          Fasle if the files should be kept after
                          encryption
            algorithm (str): The type of algorithm to be used to encrypt
                             the data
            armor (bool): True for the return type to be in ASCII string
                          False for the return type to be Crypt object

        Returns:
            None
        """
        gpg = self.gpg

        files_dir = []

        files = [f for f in os.listdir(file_path)]

        for f in files:
            files_dir.append('{}'.format(f))

        for x in files_dir:
            with open('{}{}{}'.format(file_path, os.sep, x), '{}'
                      .format('r')) as f:
                contents = f.read()
                gpg.encrypt(contents, symmetric=algorithm,
                            passphrase=symmetric_key, armor=armor,
                            recipients=None, output='{}{}{}'.format(file_path,
                            os.sep, files_dir[files_dir.index(x)]))
                # print("ok: ", crypt.ok)
                # print("status: ", crypt.status)
                # print("stderr: ", crypt.stderr)
            if delaf:
                os.rename('{}{}{}'.format(file_path, os.sep,
                          files_dir[files_dir.index(x)]), '{}{}{}'
                          .format(output, os.sep,
                          files_dir[files_dir.index(x)]))

    def encrypt_data(self, data, recipients):
        """Encrypt data.

        Method to encrypt data using the imported recipient's public
        key from user's GnuPG keyring

        Args:
            data (str): Data to be encrypted
            recipients (int): Fingerprint of recipient

        Returns:
            str: encrypted data in ASCII string
        """
        gpg = self.gpg

        encrypted_ascii_data = gpg.encrypt(data, recipients=recipients)
        # print(encrypted_ascii_data.status)
        ascii_str = str(encrypted_ascii_data)
        return ascii_str

    def sym_decrypt_files(self, symmetric_key, file_path, output, delaf=False):
        """Method to decrypt files using a symmetric key.

        Args:
            symmetric_key (str): String of passphrase to be used
            to decrypt the data
            file_path (str): Absolute file path to the files to be
            encrypted
            output (str): Absolute file path to intended file output
            delaf (bool): True if the files should be deleted after
                          decryption
                          Fasle if the files should be kept after
                          decryption
            algorithm (str): The type of algorithm that was used to
                             encrypt the data
            armor (bool): True for the return type to be in ASCII string
                          False for the return type to be Crypt object

        Returns:
            None
        """
        gpg = self.gpg

        files_dir = []

        files = [f for f in os.listdir(file_path)]

        for f in files:
            files_dir.append('{}'.format(f))

        for x in files_dir:
            with open('{}{}{}'.format(file_path, os.sep, x),
                      '{}'.format('r')) as f:
                crypt = f.read()
                # print(crypt)
                data = gpg.decrypt(crypt, passphrase=symmetric_key)
                de_data = (data.data).decode('utf-8')
                # print('\n\n\n\n--->{}<---\n\n\n'.format(de_data))
                with open('{}{}{}'.format(output, os.sep,
                          files_dir[files_dir.index(x)]),
                          '{}'.format('w')) as decrypted:
                    decrypted.write(de_data)
                # print("ok: ", data.ok)
                # print("status: ", data.status)
                # print("stderr: ", data.stderr)
            if delaf:
                os.remove('{}{}{}'.format(file_path, os.sep, x))

    def decrypt_data(self, data, passphrase):
        """Decrypt data.

        Method to decrypt data using the imported recipient's
        public key from user's GnuPG keyring

        Args:
            data (str): Data in String ASCII to be decrypted
            passphrase (str): Passphrase of the user

        Returns:
            str: Decrypted data into string
        """
        gpg = self.gpg
        passphrase = self.passphrase

        decrypted_data = gpg.decrypt(data, passphrase=passphrase)

        data = (decrypted_data.data).decode('utf-8')
        return data

    def email_to_key(self, email):
        """Email to fingerprint.

        Method to retrieve fingerprint of associated
        email address from the GnuPG keyring

        Args:
            email (str): Email address

        Returns:
            int: Fingerprint that is associated with the email
            address if it is found
            None: If no associated fingerprint is found
        """

        # Gets all available public keys in keyring
        keys = self.list_pub_keys()

        result = None

        for key in keys:  # Go through each public key
            uids = list(filter((lambda item: email in item), key['uids']))
            if uids != []:
                parts = uids[0].split(' ')
                wrapped_email = list(filter((lambda item: '<' in item), parts))
                unwrapped_email = wrapped_email[0].strip('<>')
                if unwrapped_email == email:
                    return key['fingerprint']

        return result

    def trust_key(self, fingerprint, trustlevel='TRUST_ULTIMATE'):
        """Trust public key.

        Method to trust public key that was imported to have the
        ability to encrypt data using
        that public key

        Args:
            fingerprint (int): Fingerprint of pubic key to trust
            trustlevel (str): Trust level to assign to public key

        Returns:
            None
        """
        gpg = self.gpg

        gpg.trust_keys(fingerprint, trustlevel)

    def symmetric_encrypt(self, data, passphrase,
                          algorithm='AES256', armor=True):
        """Symmetric encrypt.

        Method to encrypt data using symmmetric key encryption
        using a passphrase and encryption
        algorithm

        Args:
            data (str): String of data to be encrypted
            passphrase (str): String of passphrase to
            be used to encrypt the data
            algorithm (str): The type of algorithm to
            be used to encrypte the data
            armor (bool): True for the return type to be in ASCII string
                          False for the return type to be Crypt object

        Returns:
            str: ASCII string of encrypted data
        """
        gpg = self.gpg

        crypt = gpg.encrypt(data, symmetric=algorithm, passphrase=passphrase,
                            armor=armor, recipients=None)
        # print(crypt.status)
        return str(crypt)

    def symmetric_decrypt(self, data, passphrase):
        """Method to decrypt data that was encrypted using symmetric encryption.

        Args:
            data (str): Data in ASCII string to be decrypted
            passphrase (str): Passphrase used in the encryption

        Returns:
            str: ASCII string of decrypted data
        """
        gpg = self.gpg

        data = gpg.decrypt(data, passphrase=passphrase)
        # print(data.status)
        return (data.data).decode('utf-8')

    def gen_symm_key(self, stringLength=70):
        password_characters = string.ascii_letters + string.digits\
                              + string.punctuation
        return ''.join(random.choice(password_characters)
                       for i in range(stringLength))

    def del_pub_key(self, fingerprint):
        """Deletes public key from keyring.

        Args:
            fingerprint: Fingerprint of public key to be deleted

        Returns:
            (bool): True if the public key was deleted
                    False if the public key was not deleted
        """

        output = False
        gpg = self.gpg

        # Deletes public key
        result = gpg.delete_keys(fingerprint)

        if str(result) == 'ok':
            output = True

        return output

    def set_email(self):
        """Set email.

        Retrieve email from keyring and set the correponding email address from
        the set fingperprint

        Args:
            None

        Returns:
            None
        """

        # Retrieve gnupg object and set fingerprint
        gpg = self.gpg
        fp = self.fingerprint

        # Lists keys from keyring
        keys = gpg.list_keys()

        for key in keys:
            if key['fingerprint'] == fp:
                # Removes space from uids value and returns the items in a list
                uids = key['uids'][0].split(' ')
                # Gets the email from the items which is
                # wrapped by "<"<email@example.com>">"
                wrapped_email = list(filter((lambda item: '<' in item), uids))
                # Removes "<" and ">" from email
                email = wrapped_email[0].strip('<>')
                self.email_addr = email



#!/usr/bin/env python3
"""Creates test environment for daemon to start"""

# Standard imports
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
_EXPECTED = '{0}pattoo-shared{0}tests{0}test_pattoo_shared'.format(os.sep)
if EXEC_DIR.endswith(_EXPECTED) is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''This script is not installed in the "{0}" directory. Please fix.\
'''.format(_EXPECTED))
    sys.exit(2)

# Pattoo imports
from pattoo_shared.daemon import Daemon
from pattoo_shared.agent import Agent
from pattoo_shared.configuration import Config

# Importing MockDaemon for start test
from tests.test_pattoo_shared.test_daemon import MockDaemon, AGENT_NAME, create_agent

def start_daemon():
    """Creates a mock daemon for testing start

    Args:
        None

    Return:
        None

    """
    _agent = create_agent()

    # Setting up MockDaemon and starting process for testing
    _daemon = MockDaemon(_agent)
    _daemon.start()

def restart_daemon():
    """Creates a mock daemon for testing restart

    Args:
        None

    Return:
        None

    """
    _agent = create_agent()

    # Setting up MockDaemon and testing restart method
    _daemon = MockDaemon(_agent)
    _daemon.restart()

def daemonize_daemon():
    """Creates a mock daemon for testing daemonizing

    Args:
        None

    Return:
        None

    """
    _agent = create_agent()

    # Setting up MockDaemon and testing daemonize method
    _daemon = MockDaemon(_agent)
    _daemon._daemonize()

if __name__ == '__main__':
    command = sys.argv[1]

    if command == '--start':
        start_daemon()
    elif command == '--restart':
        restart_daemon()
    elif command == '--daemonize':
        daemonize_daemon()
    else:
        print('No command matches')



"""Functions for configuring pattoo components"""
# Standard imports
import os
import grp
import pwd
import getpass

# Import dependendices
import yaml

# Import pattoo related libraries
from pattoo_shared import files, log
from pattoo_shared.installation import shared


def create_user(user_name, directory, shell, verbose):
    """Create user and their respective group.

    Args:
        user_name: The name and group of the user being created
        directory: The home directory of the user
        shell: The shell of the user
        verbose: A boolean value that a allows the script to run in verbose
        mode

    Returns:
        None
    """
    # Ensure user has sudo privileges
    if getpass.getuser() == 'root':

        # If the group specified does not exist, it gets created
        if not group_exists(user_name):
            shared.run_script('groupadd {0}'.format(user_name), verbose)

        # If the user specified does not exist, they get created
        if not user_exists(user_name):
            shared.run_script(
                'useradd -d {1} -s {2} -g {0} {0}'.format(
                    user_name, directory, shell), verbose)
    else:
        # Die if not root
        shared.log('You are currently not running the script as root')


def group_exists(group_name):
    """Check if the group already exists.

    Args:
        group_name: The name of the group

    Returns
        True if the group exists and False if it does not
    """
    try:
        # Gets group name
        grp.getgrnam(group_name)
        return True
    except KeyError:
        return False


def read_config(filepath, default_config):
    """Read configuration file and replace default values.

    Args:
        filepath: Name of configuration file
        default_config: Default configuration dict

    Returns:
        config: Dict of configuration

    """
    # Read config
    if os.path.isfile(filepath) is True:
        try:
            f_handle = open(filepath, 'r')
        except PermissionError:
            log.log2die_safe(1078, '''\
Insufficient permissions for reading the file:{}'''.format(filepath))
        else:
            with f_handle:
                yaml_string = f_handle.read()
                config = yaml.safe_load(yaml_string)
    else:
        config = default_config

    return config


def check_config(config_file, config_dict):
    """Ensure agent configuration exists.

    Args:
        config: The name of the configuration file
        config_dict: A dictionary containing the primary configuration keys
        and a list of the secondary keys

    Returns:
        None

    """
    # Initialize key variables
    config_directory = os.environ['PATTOO_CONFIGDIR']

    # Print Status
    print('??: Checking configuration parameters.')

    # Retrieve config dict
    config = files.read_yaml_file(config_file)

    # Check main keys
    for primary in config_dict:
        if primary not in config:
            log_message = ('''\
Section "{}" not found in configuration file {} in directory {}. Please fix.\
    '''.format(primary, config_file, config_directory))
            log.log2die_safe(1055, log_message)

    # Print Status
    print('OK: Configuration parameter check passed.')


def user_exists(user_name):
    """Check if the user already exists.

    Args:
        user_name: The name of the user

    Returns
        True if the user exists and False if it does not

    """
    try:
        # Gets user name
        pwd.getpwnam(user_name)
        return True
    except KeyError:
        return False


def pattoo_config(file_name, config_directory, config_dict):
    """Create configuration file.

    Args:
        file_name: Name of the configuration file without its file extension
        config_directory: Full path to the configuration directory
        config_dict: A dictionary containing the configuration values.

    Returns:
        The path to the configuration file

    """
    # Initialize key variables
    config_file = os.path.join(config_directory, '{}.yaml'.format(file_name))

    # Say what we are doing
    print('\nConfiguring {} file.\n'.format(config_file))

    # Get configuration
    config = read_config(config_file, config_dict)

    # Check validity of directories, if any
    if bool(config) is False:
        # Set default
        config = config_dict

    # Iterate over config dict
    for key, value in sorted(config.items()):
        if type(value) == dict:
            for secondary_key in value:
                if 'directory' in secondary_key:
                    if os.sep not in value.get(secondary_key):
                        log.log2die_safe(
                            1019, '{} is an invalid directory'.format(value))

                    # Attempt to create directory
                    full_directory = os.path.expanduser(value.get(secondary_key))
                    if os.path.isdir(full_directory) is False:
                        print('Creating: {}'.format(full_directory))
                        files.mkdir(full_directory)

                    # Recursively set file ownership to pattoo user and group
                    if getpass.getuser() == 'root':
                        shared.chown(full_directory)

        # Write file
        try:
            f_handle = open(config_file, 'w')
        except PermissionError:
            log.log2die(1076, '''\
Insufficient permissions for creating the file:{}'''.format(config))
        else:
            with f_handle:
                yaml.dump(config, f_handle, default_flow_style=False)

    return config_file


def configure_component(component_name, config_dir, config_dict):
    """Configure individual pattoo related components and check configuration.

    Args:
        component_name: The file name for the component being configured
        config_dir: The directory with the configuration files
        config_dict: A dictionary containing the configuration values

    Returns:
        None
    """
    # Create configuration
    config_file = pattoo_config(
                                component_name,
                                config_dir,
                                config_dict)

    # Check if configuration is valid
    check_config(config_file, config_dict)
